@@ -1,6 +1,8 @@
-__version__ = "2.0.0"
+__version__ = "2.1.7"
 
 from s3parq.fetch_parq import fetch
 from s3parq.fetch_parq import fetch_diff
+from s3parq.fetch_parq import get_all_files_list
 from s3parq.fetch_parq import get_max_partition_value
 from s3parq.publish_parq import publish
+from s3parq.session_helper import SessionHelper
