@@ -2,6 +2,8 @@ import ast
 import boto3
 from collections import OrderedDict
 import datetime
+from distutils.util import strtobool
+import logging
 import operator
 from typing import Dict, List, Any
 import multiprocessing as mp
@@ -12,25 +14,6 @@ import pyarrow as pa
 import pyarrow.parquet as pq
 from .s3_naming_helper import S3NamingHelper
 
-''' S3 Parquet to Dataframe Fetcher.
-This class handles the portion of work that will return a concatenated 
-dataframe based on the partition filters the specified dataset.
-
-Required kwargs:
-    bucket (str): S3 Bucket name
-    key (str): S3 key that leads to the desired dataset
-    filters (List(Filter)): filters to be applied to the Dataset Partitions
-        Filters have the fields:
-            partition (str):
-                Partition to be applied to
-            comparison (str):
-                Comparison function - one of: [ == , != , > , < , >= , <= ]
-            values (List(any)): 
-                Values to compare to - must match partition data type
-                May not use multiple values with the '<', '>' comparisons
-'''
-
-
 # Filter
 # class Filter(NamedTuple):
 #    partition: str
@@ -60,9 +43,22 @@ NON_NUM_TYPES = [
 ]
 
 
+logger = logging.getLogger(__name__)
+
+
 def get_all_partition_values(bucket: str, key: str, partition: str) -> iter:
-    """retruns all values, correctly typed, for a given partition IN NO ORDER."""
-    all_files = _get_all_files_list(bucket, key)
+    """ Returns all values, correctly typed, for a given partition IN NO ORDER
+
+    Args:
+        bucket (str): S3 Bucket name
+        key (str): S3 key that leads to the desired dataset
+        partition (str): The partition in the dataset to parse the values out of
+
+    Returns:
+        An iterable of all the partition values
+    """
+
+    all_files = get_all_files_list(bucket, key)
 
     if not all_files:
         return []
@@ -75,12 +71,23 @@ def get_all_partition_values(bucket: str, key: str, partition: str) -> iter:
 
 
 def get_diff_partition_values(bucket: str, key: str, partition: str, values_to_diff: iter, reverse: bool = False) -> iter:
-    """ returns all the partition values in bucket/key that are not in values_to_diff.
-        ARGS:
-            values_to_diff: the iterable of values to compare the partition values to
-            reverse: if True, will look for the values in values_to_diff that are not in partition values (basically backwards)    
-     """
-    all_files = _get_all_files_list(bucket, key)
+    """ Returns all the partition values in the dataset at the bucket/key
+    that are not in values_to_diff
+
+    Args:
+        bucket (str): S3 Bucket name
+        key (str): S3 key that leads to the desired dataset
+        partition (str): The partition in the dataset to parse the values out of
+        values_to_diff (iter): The iterable set of values to compare the partition values to
+        reverse (bool, Optional): Determines if the operation should be inversed,
+            if True it will look for the values in values_to_diff that are not
+            in the partition values (basically backwards). Defaults to False
+
+    Returns:
+        An iterable of all the partition values that are not in values_to_diff
+            and vice-versa if reverse is True
+    """
+    all_files = get_all_files_list(bucket, key)
 
     if not all_files:
         if reverse:
@@ -112,12 +119,20 @@ def get_diff_partition_values(bucket: str, key: str, partition: str, values_to_d
 
 
 def get_max_partition_value(bucket: str, key: str, partition: str) -> any:
-    ''' Returns the max value of the specified partition
-    in the data type from the metadata.
-    '''
+    """ Returns the max partition value in the dataset at the bucket/key
+    NOTE: This functionality cannot occur on non-numerical datatypes!
+
+    Args:
+        bucket (str): S3 Bucket name
+        key (str): S3 key that leads to the desired dataset
+        partition (str): The partition in the dataset to parse the values out of
+
+    Returns:
+        The max partition value, of the matching partition datatype
+    """
     S3NamingHelper().validate_bucket_name(bucket_name=bucket)
 
-    all_files = _get_all_files_list(bucket=bucket, key=key)
+    all_files = get_all_files_list(bucket=bucket, key=key)
 
     if not all_files:
         return None
@@ -134,14 +149,40 @@ def get_max_partition_value(bucket: str, key: str, partition: str) -> any:
     return max([convert_type(val, partition_dtype) for val in partition_values])
 
 
-def fetch(bucket: str, key: str, filters: List[type(Filter)] = {}, parallel: bool = True):
-    ''' Access function to kick off all bits and return result. '''
+def fetch(bucket: str, key: str, filters: List[type(Filter)] = {}, parallel: bool = True) -> pd.DataFrame:
+    """ S3 Parquet to Dataframe Fetcher.
+    This function handles the portion of work that will return a concatenated 
+    dataframe based on the partition filters of the specified dataset.
+
+    Args:
+        bucket (str): S3 Bucket name
+        key (str): S3 key that leads to the desired dataset
+        filters (List(Filter), Optional): filters to be applied to the Dataset Partitions
+            Filters have the fields:
+
+                - partition (str):
+                    The published partition to be applied to
+                - comparison (str):
+                    Comparison function - one of: [ == , != , > , < , >= , <= ]
+                - values (List(any)): 
+                    Values to compare to - must match partition data type
+                    May not use multiple values with the '<', '>' comparisons
+        parallel (bool, Optional):
+            Determines if multiprocessing should be used, defaults to True
+            Whether this is more or less efficient is dataset dependent,
+                smaller datasets run much quicker without parallel
+
+    Returns:
+        A pandas dataframe of the filtered results from S3
+    """
+
     _validate_filter_rules(filters)
     S3NamingHelper().validate_bucket_name(bucket)
 
-    all_files = _get_all_files_list(bucket, key)
+    all_files = get_all_files_list(bucket, key)
 
     if not all_files:
+        logger.debug(f"No files present under : {key} :, returning empty DataFrame")
         return pd.DataFrame()
 
     partition_metadata = _get_partitions_and_types(all_files[0], bucket)
@@ -167,6 +208,7 @@ def fetch(bucket: str, key: str, filters: List[type(Filter)] = {}, parallel: boo
     # if there is no data matching the filters, return an empty DataFrame
     # with correct headers and type
     if len(files_to_load) < 1:
+        logger.debug(f"Dataset filters left no matching values, returning empty dataframe with matching headers")
         sacrifical_files = [all_files[0]]
         sacrifical_frame = _get_filtered_data(
             bucket=bucket, paths=sacrifical_files, partition_metadata=partition_metadata, parallel=parallel)
@@ -177,14 +219,27 @@ def fetch(bucket: str, key: str, filters: List[type(Filter)] = {}, parallel: boo
 
 
 def fetch_diff(input_bucket: str, input_key: str, comparison_bucket: str, comparison_key: str, partition: str, reverse: bool = False, parallel: bool = True) -> pd.DataFrame:
-    ''' Returns a dataframe of whats in the input dataset but not the comparison dataset by the specified partition
-        ARGS:
-            input_bucket (str): the bucket of the dataset to start from
-            input_key (str): the key to the dataset to start from
-            comparison_bucket (str): the bucket of the dataset to compare against
-            comparison_key (str): the key to the dataset to compare against
-            partition (str): the partition whos values to compare
-    '''
+    """ Returns a dataframe of whats in the input dataset but not the comparison 
+    dataset by the specified partition.
+
+    Args:
+        input_bucket (str): The bucket of the dataset to start from
+        input_key (str): The key to the dataset to start from
+        comparison_bucket (str): The bucket of the dataset to compare against
+        comparison_key (str): The key to the dataset to compare against
+        partition (str): The partition in the dataset to compare the values of
+        reverse (bool, Optional): Determines if the operation should be inversed,
+            if True it will look for the values in comparison that are not 
+            in the input (basically backwards). Defaults to False
+        parallel (bool, Optional):
+            Determines if multiprocessing should be used, defaults to True.
+            Whether this is more or less efficient is dataset dependent,
+            smaller datasets run much quicker without parallel
+
+    Returns:
+        A dataframe of all values in the input but not the comparison,
+            if reverse=True then vice-versa
+    """
     S3NamingHelper().validate_bucket_name(input_bucket)
     S3NamingHelper().validate_bucket_name(comparison_bucket)
 
@@ -205,6 +260,8 @@ def fetch_diff(input_bucket: str, input_key: str, comparison_bucket: str, compar
         "values": diff_values
     }]
 
+    logger.debug(f"Fetching difference, looking for values : {diff_values} : under partition : {partition} ")
+
     if reverse:
         return fetch(bucket=comparison_bucket, key=comparison_key, filters=filters, parallel=parallel)
     else:
@@ -212,10 +269,18 @@ def fetch_diff(input_bucket: str, input_key: str, comparison_bucket: str, compar
 
 
 def convert_type(val: Any, dtype: str) -> Any:
-    """ converts a value to the given datatype"""
-    if dtype == 'string':
+    """ Converts the given value to the given datatype
+
+    Args:
+        val (Any): The value to convert
+        dtype (str): The type to attempt to convert to
+
+    Returns:
+        The value parsed into the new dtype
+    """
+    if dtype == 'string' or dtype == 'str':
         return str(val)
-    elif dtype == 'integer':
+    elif dtype == 'integer' or dtype == 'int':
         return int(val)
     elif dtype == 'float':
         return float(val)
@@ -224,11 +289,19 @@ def convert_type(val: Any, dtype: str) -> Any:
             val, '%Y-%m-%d %H:%M:%S')
     elif dtype == 'category':
         return pd.Category(val)
-    elif dtype == 'bool':
-        return bool(val)
+    elif dtype == 'bool' or dtype == 'boolean':
+        return bool(strtobool(val))
 
 
-def dtype_to_pandas_dtype(dtype: str):
+def dtype_to_pandas_dtype(dtype: str) -> str:
+    """ Matches the given dtype to its pandas equivalent, if one exists
+
+    Args:
+        dtype (str): The type to attempt to match
+
+    Returns:
+        The pandas version of the dtype if one exists, otherwise the original
+    """
     if dtype == "integer":
         dtype = "int"
     elif dtype == "string":
@@ -239,31 +312,19 @@ def dtype_to_pandas_dtype(dtype: str):
     return dtype
 
 
-def _get_partitions_and_types(first_file_key: str, bucket):
-    ''' Fetch a list of all the partitions actually there and their 
-    datatypes. List may be different than passed list if not being used
-    for filtering on.
-    - This is pulled from the metadata. It is assumed that this package
-        is being used with its own publish method.
-    '''
-    parts_and_types = []
-    s3_client = boto3.client('s3')
-
-    first_file = s3_client.head_object(
-        Bucket=bucket,
-        Key=first_file_key
-    )
-
-    partition_metadata = ast.literal_eval(
-        first_file['Metadata']['partition_data_types'])
-
-    return partition_metadata
-
-
-def _get_all_files_list(bucket, key) -> list:
-    ''' Get a list of all files to get all partitions values.
+def get_all_files_list(bucket: str, key: str) -> list:
+    """ Get a list of all files to get all partitions values.
     Necesarry to catch all partition values for non-filtered partiions.
-    '''
+    NOTE: This paginates across as many as are matching keys in the bucket;
+        be mindful of any costs in extra large buckets without a specific key.
+
+    Args:
+        bucket (str): S3 bucket to search in
+        key (str): S3 key to the dataset to check
+
+    Returns:
+        A list of the keys of all objects in the bucket/key that end with .parquet
+    """
     objects_in_bucket = []
     s3_client = boto3.client('s3')
     paginator = s3_client.get_paginator('list_objects')
@@ -281,10 +342,46 @@ def _get_all_files_list(bucket, key) -> list:
     return objects_in_bucket
 
 
+def _get_partitions_and_types(first_file_key: str, bucket: str) -> dict:
+    """ Fetch a list of all the partitions actually there and their 
+    datatypes. List may be different than passed list if not being used
+    for filtering on.
+    NOTE: This is pulled from the metadata. It is assumed that this package
+        is being used with its own publish method.
+
+    Args:
+        first_file_key (str): The key to the first file in the dataset, to get
+            the metadata off of
+        bucket (str): The S3 bucket to run in
+
+    Returns:
+        The parsed metadata from heading the first file
+    TODO
+    """
+    parts_and_types = []
+    s3_client = boto3.client('s3')
+
+    first_file = s3_client.head_object(
+        Bucket=bucket,
+        Key=first_file_key
+    )
+
+    partition_metadata = ast.literal_eval(
+        first_file['Metadata']['partition_data_types'])
+
+    return partition_metadata
+
+
 def _parse_partitions_and_values(file_paths: List[str], key: str) -> dict:
-    ''' Take the list of all the file keys and return a dict of the
-    partitions with arrays of their values.
-    '''
+    """ Parses the string keys into a usable dict of the parts and values
+
+    Args:
+        file_paths (List[str]): The list of all files to parse out
+        key (str): S3 key to the root of the dataset of the files
+
+    Returns:
+        A dictionary of all partitions with their values
+    """
     # TODO: find more neat/efficient way to do this
     parts = OrderedDict()
     key_len = len(key)
@@ -305,9 +402,16 @@ def _parse_partitions_and_values(file_paths: List[str], key: str) -> dict:
     return parts
 
 
-def _get_partition_value_data_types(parsed_parts: dict, part_types: dict):
-    ''' Convert the collected values to their python data types for use.
-    '''
+def _get_partition_value_data_types(parsed_parts: dict, part_types: dict) -> dict:
+    """ Uses the partitions with their known types to parse them out
+
+    Args:
+        parsed_parts (dict): A dictionary of all partitions with their values
+        part_types (dict): A dictionary of all partitions to their datatypes
+
+    Returns:
+        A dictionary of all partitions with their values parsed into the correct datatype
+    """
     for part, values in parsed_parts.items():
         part_type = part_types[part]
         if (part_type == 'string') or (part_type == 'category'):
@@ -321,16 +425,24 @@ def _get_partition_value_data_types(parsed_parts: dict, part_types: dict):
                 map(lambda s: datetime.datetime.strptime(s, "%Y-%m-%d %H:%M:%S"), values))
         elif (part_type == 'bool') or (part_type == 'boolean'):
             parsed_parts[part] = set(map(bool, values))
+        else:
+            logger.debug(f"Unknown partition type : {part_type} :, leaving as a string")
 
     return parsed_parts
 
-# TODO: Neaten up?
 
-
-def _get_filtered_key_list(typed_parts: dict, filters, key) -> List[str]:
-    ''' Create list of all "paths" to files after the filtered partitions
+def _get_filtered_key_list(typed_parts: dict, filters: List[type(Filter)], key: str) -> List[str]:
+    """ Create list of all "paths" to files after the filtered partitions
     are set ie all non-matching partitions are excluded.
-    '''
+
+    Args:
+        typed_parts (dict): A dictionary of all partitions with their values
+        filters (List[type(Filter)]): A dictionary of all partitions to their datatypes
+        key (str): S3 key to the base root of the dataset
+
+    Returns:
+        A list of object keys that have been filtered by partition values
+    """
     filter_keys = []
     matched_parts = OrderedDict()
     matched_parts.keys = typed_parts.keys()
@@ -348,7 +460,7 @@ def _get_filtered_key_list(typed_parts: dict, filters, key) -> List[str]:
             matched_parts[part] = matched_part_vals.copy()
         else:
             matched_parts[part] = part_values
-            
+
     def construct_paths(matched_parts, previous_fil_keys: List[str]) -> None:
         if len(matched_parts) > 0:
             part = matched_parts.popitem(last=False)
@@ -370,32 +482,56 @@ def _get_filtered_key_list(typed_parts: dict, filters, key) -> List[str]:
     return filter_keys[0]
 
 
-def _get_filtered_data(bucket: str, paths: list, partition_metadata, parallel=True) -> pd.DataFrame:
-    ''' Pull all filtered parquets down and return a dataframe.
-    '''
+def _get_filtered_data(bucket: str, paths: List[str], partition_metadata: dict, parallel: bool = True) -> pd.DataFrame:
+    """ Gets the data based on the filtered object key list. Concatenates all 
+    the separate parquet files.
+
+    Args:
+        bucket (str): S3 bucket to fetch from
+        paths (List[str]): A list of all the object keys of the parquet files
+        partition_metadata (dict): A dictionary of all partitions to their datatypes
+        parallel (bool, Optional):
+            Determines if multiprocessing should be used, defaults to True.
+            Whether this is more or less efficient is dataset dependent,
+            smaller datasets run much quicker without parallel
+
+    Returns:
+        The dataframe of all the parquet files from the given keys
+    """
 
     temp_frames = []
 
-    def append_to_temp(frame):
+    def append_to_temp(frame: pd.DataFrame):
         temp_frames.append(frame)
 
     if parallel:
         with get_context("spawn").Pool() as pool:
             for path in paths:
+                logger.debug(f"[Parallel] Fetching parquet file to append from : {path} ")
                 append_to_temp(
                     pool.apply_async(_s3_parquet_to_dataframe, args=(bucket, path, partition_metadata)).get())
             pool.close()
             pool.join()
     else:
         for path in paths:
+            logger.debug(f"Fetching parquet file to append from : {path} ")
             append_to_temp(_s3_parquet_to_dataframe(
                 bucket, path, partition_metadata))
 
-    return pd.concat(temp_frames)
+    return pd.concat(temp_frames, ignore_index=True)
 
 
-def _s3_parquet_to_dataframe(bucket: str, key: str, partition_metadata) -> pd.DataFrame:
-    """ grab a parquet file from s3 and convert to pandas df, add it to the destination"""
+def _s3_parquet_to_dataframe(bucket: str, key: str, partition_metadata: dict) -> pd.DataFrame:
+    """ Grab a parquet file from s3 and convert to pandas df, add it to the destination
+
+    Args:
+        bucket (str): S3 bucket to fetch from
+        key (key): The full object path of the parquet file to read
+        partition_metadata (dict): A dictionary of all partitions to their datatypes
+
+    Returns:
+        Dataframe from the key parquet, with partitions repopulated
+    """
     s3 = s3fs.S3FileSystem()
     uri = f"{bucket}/{key}"
     table = pq.ParquetDataset(uri, filesystem=s3)
@@ -406,8 +542,16 @@ def _s3_parquet_to_dataframe(bucket: str, key: str, partition_metadata) -> pd.Da
     return frame
 
 
-def _repopulate_partitions(partition_string: str, partition_metadata) -> tuple:
-    """ for each val in the partition string creates a list that can be added back into the dataframe"""
+def _repopulate_partitions(partition_string: str, partition_metadata: dict) -> tuple:
+    """ For each val in the partition string creates a list that can be added back into the dataframe
+
+    Args:
+        partition_string (str): S3 bucket to fetch from
+        partition_metadata (dict): A dictionary of all partitions to their datatypes
+
+    Returns:
+        Dataframe from the key parquet, with partitions repopulated
+    """
     raw = partition_string.split('/')
     partitions = {}
     for string in raw:
@@ -421,9 +565,15 @@ def _repopulate_partitions(partition_string: str, partition_metadata) -> tuple:
 
 
 def _validate_filter_rules(filters: List[type(Filter)]) -> None:
-    ''' Validate that the filters are the correct format and follow basic
-    comparison rules.
-    '''
+    """ Validate that the filters are the correct format and follow basic
+    comparison rules, otherwise throw a ValueError
+
+    Args:
+        filters (List[type(Filter)]): List of filters to validate
+
+    Returns:
+        None
+    """
 
     single_value_comparisons = [
         ">",
@@ -443,10 +593,18 @@ def _validate_filter_rules(filters: List[type(Filter)]) -> None:
                 f"Comparison {f['comparison']} can only be used with one filter value.")
 
 
-def _validate_matching_filter_data_type(part_types, filters) -> None:
-    ''' Validate that the filters passed are matching to the partitions'
-    listed datatypes. This includes validating comparisons too.
-    '''
+def _validate_matching_filter_data_type(part_types, filters: List[type(Filter)]) -> None:
+    """ Validate that the filters passed are matching to the partitions'
+    listed datatypes, otherwise throw a ValueError. 
+    This includes validating comparisons too.
+
+    Args:
+        part_types (dict): A dictionary of all partitions to their datatypes
+        filters (List[type(Filter)]): List of filters to validate
+
+    Returns:
+        None
+    """
     num_comparisons = [
         ">",
         "<",
